@@ -589,4 +589,8 @@
 # define DEBUG_MEMORY_POOLS 1
 #endif
 
+#ifndef STKTABLE_MAX_UPDATES_AT_ONCE
+#define STKTABLE_MAX_UPDATES_AT_ONCE 100
+#endif /* STKTABLE_MAX_UPDATES_AT_ONCE */
+
 #endif /* _HAPROXY_DEFAULTS_H */
