@@ -52,6 +52,7 @@ enum pr_mode {
 	PR_MODE_CLI,
 	PR_MODE_SYSLOG,
 	PR_MODE_PEERS,
+	PR_MODE_SPOP,
 	PR_MODES
 } __attribute__((packed));
 
@@ -191,17 +192,18 @@ enum PR_SRV_STATE_FILE {
 #define PR_RE_403                 0x00000010 /* Retry if we got a 403 */
 #define PR_RE_404                 0x00000020 /* Retry if we got a 404 */
 #define PR_RE_408                 0x00000040 /* Retry if we got a 408 */
-#define PR_RE_425                 0x00000080 /* Retry if we got a 425 */
-#define PR_RE_429                 0x00000100 /* Retry if we got a 429 */
-#define PR_RE_500                 0x00000200 /* Retry if we got a 500 */
-#define PR_RE_501                 0x00000400 /* Retry if we got a 501 */
-#define PR_RE_502                 0x00000800 /* Retry if we got a 502 */
-#define PR_RE_503                 0x00001000 /* Retry if we got a 503 */
-#define PR_RE_504                 0x00002000 /* Retry if we got a 504 */
+#define PR_RE_421                 0x00000080 /* Retry if we got a 421 */
+#define PR_RE_425                 0x00000100 /* Retry if we got a 425 */
+#define PR_RE_429                 0x00000200 /* Retry if we got a 429 */
+#define PR_RE_500                 0x00000400 /* Retry if we got a 500 */
+#define PR_RE_501                 0x00000800 /* Retry if we got a 501 */
+#define PR_RE_502                 0x00001000 /* Retry if we got a 502 */
+#define PR_RE_503                 0x00002000 /* Retry if we got a 503 */
+#define PR_RE_504                 0x00004000 /* Retry if we got a 504 */
 #define PR_RE_STATUS_MASK         (PR_RE_401 | PR_RE_403 | PR_RE_404 | \
-                                   PR_RE_408 | PR_RE_425 | PR_RE_429 | \
-                                   PR_RE_500 | PR_RE_501 | PR_RE_502 | \
-                                   PR_RE_503 | PR_RE_504)
+                                   PR_RE_408 | PR_RE_421 | PR_RE_425 | \
+                                   PR_RE_429 | PR_RE_500 | PR_RE_501 | \
+                                   PR_RE_502 | PR_RE_503 | PR_RE_504)
 /* 0x00000800, 0x00001000, 0x00002000, 0x00004000 and 0x00008000 unused,
  * reserved for eventual future status codes
  */
@@ -307,6 +309,9 @@ struct proxy {
 		unsigned int inspect_delay;     /* inspection delay */
 		struct list inspect_rules;      /* inspection rules */
 	} tcp_rep;
+#ifdef USE_QUIC
+	struct list quic_init_rules;		/* quic-initial rules */
+#endif
 	struct server *srv, defsrv;		/* known servers; default server configuration */
 	struct lbprm lbprm;			/* load-balancing parameters */
 	int srv_act, srv_bck;			/* # of servers eligible for LB (UP|!checked) AND (enabled+weight!=0) */
@@ -375,7 +380,7 @@ struct proxy {
 	struct buffer log_tag;                   /* override default syslog tag */
 	struct ist header_unique_id; 		/* unique-id header */
 	struct lf_expr format_unique_id;        /* unique-id format */
-	int to_log;				/* things to be logged (LW_*) */
+	int to_log;				/* things to be logged (LW_*), special value LW_LOGSTEPS == follow log-steps */
 	int nb_req_cap, nb_rsp_cap;		/* # of headers to be captured */
 	struct cap_hdr *req_cap;		/* chained list of request headers to be captured */
 	struct cap_hdr *rsp_cap;		/* chained list of response headers to be captured */
@@ -410,7 +415,7 @@ struct proxy {
 	int no_options2;			/* PR_O2_* */
 
 	struct {
-		char *file;			/* file where the section appears */
+		const char *file;		/* file where the section appears */
 		struct eb32_node id;		/* place in the tree of used IDs */
 		int line;			/* line where the section appears */
 		struct eb_root used_listener_id;/* list of listener IDs in use */
@@ -420,9 +425,12 @@ struct proxy {
 		struct list listeners;		/* list of listeners belonging to this frontend */
 		struct list errors;             /* list of all custom error files */
 		struct arg_list args;           /* sample arg list that need to be resolved */
-		unsigned int refcount;          /* refcount on this proxy (only used for default proxy for now) */
 		struct ebpt_node by_name;       /* proxies are stored sorted by name here */
 		struct list lf_checks;          /* list of logformats found in the proxy section that needs to be checked during postparse */
+		struct eb_root log_steps;       /* tree of log origins where log should be generated during request handling */
+		const char *file_prev;          /* file of the previous instance found with the same name, or NULL */
+		int line_prev;                  /* line of the previous instance found with the same name, or 0 */
+		unsigned int refcount;          /* refcount on this proxy (only used for default proxy for now) */
 	} conf;					/* config information */
 	struct http_ext *http_ext;	        /* http ext options */
 	struct eb_root used_server_addr;        /* list of server addresses in use */
@@ -439,7 +447,7 @@ struct proxy {
 		char *myhostname;		/* Identity to use in HELO command sent to mailer */
 		int level;			/* Maximum syslog level of messages to send
 						 * email alerts for */
-		int set;			/* True if email_alert settings are present */
+		int flags;                      /* check mailers.h for available flags */
 		struct email_alertq *queues;	/* per-mailer alerts queues */
 	} email_alert;
 
@@ -511,8 +519,10 @@ struct redirect_rule {
 	struct lf_expr rdr_fmt;
 	int code;
 	unsigned int flags;
-	int cookie_len;
-	char *cookie_str;
+	union {
+		struct ist str;     /* the cookie is a string  */
+		struct lf_expr fmt; /* or a log-format string (possible for set-cookie only) */
+	} cookie;
 };
 
 /* some of the most common options which are also the easiest to handle */
