@@ -82,6 +82,21 @@
  */
 extern int strlcpy2(char *dst, const char *src, int size);
 
+/*
+ * portable equivalent to POSIX strnlen():
+ * returns the number of bytes in the string pointed to by <s>, excluding
+ * the terminating null byte, but at most <maxlen>. The function does not
+ * look at characters passed <maxlen>.
+ */
+static inline size_t strnlen2(const char *s, size_t maxlen)
+{
+	size_t len;
+
+	for (len = 0; len < maxlen && s[len]; len++)
+		;
+	return len;
+}
+
 /*
  * This function simply returns a locally allocated string containing
  * the ascii representation for number 'n' in decimal.
@@ -100,6 +115,18 @@ static inline const char *ultoa(unsigned long n)
 	return ultoa_r(n, itoa_str[0], sizeof(itoa_str[0]));
 }
 
+/* file names management */
+const char *copy_file_name(const char *name);
+void free_all_file_names();
+
+/* This is only used as a marker for call places where a free() of a file name
+ * is expected to be performed, and to reset the pointer.
+ */
+static inline void drop_file_name(const char **name)
+{
+	*name = NULL;
+}
+
 /*
  * unsigned long long ASCII representation
  *
@@ -290,7 +317,7 @@ static inline int is_idchar(char c)
  */
 struct sockaddr_storage *str2sa_range(const char *str, int *port, int *low, int *high, int *fd,
                                       struct protocol **proto, struct net_addr_type *sa_type,
-                                      char **err, const char *pfx, char **fqdn, unsigned int opts);
+                                      char **err, const char *pfx, char **fqdn, int *alt, unsigned int opts);
 
 
 /* converts <addr> and <port> into a string representation of the address and port. This is sort
@@ -615,7 +642,26 @@ extern time_t my_timegm(const struct tm *tm);
  * <ret> is left untouched.
  */
 extern const char *parse_time_err(const char *text, unsigned *ret, unsigned unit_flags);
-extern const char *parse_size_err(const char *text, unsigned *ret);
+extern const char *parse_size_ui(const char *text, unsigned *ret);
+extern const char *parse_size_ull(const char *text, ullong *ret);
+
+/* Parse a size from <_test> into <_ret> which must be compatible with a
+ * uint or ullong. The return value is a pointer to the first unparsable
+ * character (if any) or NULL if everything's OK.
+ */
+#define parse_size_err(_text, _ret) ({			\
+	const char *_err;				\
+	if (sizeof(*(_ret)) > sizeof(int)) {		\
+		unsigned long long _tmp;		\
+		_err = parse_size_ull(_text, &_tmp);	\
+		*_ret = _tmp;				\
+	} else {					\
+		unsigned int _tmp;			\
+		_err = parse_size_ui(_text, &_tmp);	\
+		*_ret = _tmp;				\
+	}						\
+	_err;						\
+})
 
 /*
  * Parse binary string written in hexadecimal (source) and store the decoded
@@ -714,7 +760,11 @@ static inline int is_inet_addr(const struct sockaddr_storage *addr)
  */
 static inline int is_addr(const struct sockaddr_storage *addr)
 {
-	if (addr->ss_family == AF_UNIX || addr->ss_family == AF_CUST_SOCKPAIR)
+	/* WT: ideally we should use real_family(addr->ss_family) here, but we
+	 * have so few custom addresses that it's simple enough to test them all.
+	 */
+	if (addr->ss_family == AF_UNIX || addr->ss_family == AF_CUST_ABNS ||
+	    addr->ss_family == AF_CUST_ABNSZ || addr->ss_family == AF_CUST_SOCKPAIR)
 		return 1;
 	else
 		return is_inet_addr(addr);
@@ -753,7 +803,16 @@ static inline int get_addr_len(const struct sockaddr_storage *addr)
 	case AF_INET6:
 		return sizeof(struct sockaddr_in6);
 	case AF_UNIX:
+	case AF_CUST_ABNS:
 		return sizeof(struct sockaddr_un);
+	case AF_CUST_ABNSZ:
+		{
+			const struct sockaddr_un *un = (struct sockaddr_un *)addr;
+
+			/* stop at first NULL-byte */
+			return offsetof(struct sockaddr_un, sun_path) + 1 +
+			       strnlen2(un->sun_path + 1, sizeof(un->sun_path) - 1);
+		}
 	}
 	return 0;
 }
@@ -1043,8 +1102,6 @@ static inline unsigned long long rdtsc()
  * The caller is responsible for freeing the <err> and <str> copy
  * memory area using free()
  */
-struct list;
-int list_append_word(struct list *li, const char *str, char **err);
 
 int dump_text(struct buffer *out, const char *buf, int bsize);
 int dump_binary(struct buffer *out, const char *buf, int bsize);
@@ -1056,6 +1113,7 @@ void dump_area_with_syms(struct buffer *output, const void *base, const void *ad
 void dump_hex(struct buffer *out, const char *pfx, const void *buf, int len, int unsafe);
 int may_access(const void *ptr);
 const void *resolve_sym_name(struct buffer *buf, const char *pfx, const void *addr);
+const void *resolve_dso_name(struct buffer *buf, const char *pfx, const void *addr);
 const char *get_exec_path(void);
 void *get_sym_curr_addr(const char *name);
 void *get_sym_next_addr(const char *name);
@@ -1201,7 +1259,7 @@ static inline void update_char_fingerprint(uint8_t *fp, char prev, char curr)
 	switch (prev) {
 	case 0:         from = 28; break; // begin
 	case 'a'...'z': from = prev - 'a' + 1; break;
-	case 'A'...'Z': from = tolower(prev) - 'a' + 1; break;
+	case 'A'...'Z': from = tolower((unsigned char)prev) - 'a' + 1; break;
 	case '0'...'9': from = 27; break;
 	default:        from = 28; break;
 	}
@@ -1209,7 +1267,7 @@ static inline void update_char_fingerprint(uint8_t *fp, char prev, char curr)
 	switch (curr) {
 	case 0:         to = 28; break; // end
 	case 'a'...'z': to = curr - 'a' + 1; break;
-	case 'A'...'Z': to = tolower(curr) - 'a' + 1; break;
+	case 'A'...'Z': to = tolower((unsigned char)curr) - 'a' + 1; break;
 	case '0'...'9': to = 27; break;
 	default:        to = 28; break;
 	}
@@ -1218,6 +1276,23 @@ static inline void update_char_fingerprint(uint8_t *fp, char prev, char curr)
 	fp[32 * from + to]++;
 }
 
+/* checks that the numerical argument, if passed without units and is non-zero,
+ * is at least as large as value <min>. It returns 1 if the value is too small,
+ * otherwise zero. This is used to warn about the use of small values without
+ * units.
+ */
+static inline int warn_if_lower(const char *text, long min)
+{
+	int digits;
+	long value;
+
+	digits = strspn(text, "0123456789");
+	if (digits < strlen(text))
+		return 0; // there are non-digits here.
+
+	value = atol(text);
+	return value && value < min;
+}
 
 /* compare the current OpenSSL version to a string */
 int openssl_compare_current_version(const char *version);
@@ -1228,4 +1303,12 @@ int openssl_compare_current_name(const char *name);
 void vma_set_name(void *addr, size_t size, const char *type, const char *name);
 void vma_set_name_id(void *addr, size_t size, const char *type, const char *name, unsigned int id);
 
+/* cfgparse helpers */
+char *fgets_from_mem(char* buf, int size, const char **position, const char *end);
+
+/* helpers to backup/clean/restore process env */
+int backup_env(void);
+int clean_env(void);
+int restore_env(void);
+
 #endif /* _HAPROXY_TOOLS_H */
