@@ -65,12 +65,55 @@ static int dns_connect_nameserver(struct dns_nameserver *ns)
 			 ns->counters->pid, ns->id);
 		return -1;
 	}
-	if (connect(fd, (struct sockaddr*)&dgram->addr.to, get_addr_len(&dgram->addr.to)) == -1) {
-		send_log(NULL, LOG_WARNING,
-			 "DNS : section '%s': can't connect socket for nameserver '%s'.\n",
-			 ns->counters->id, ns->id);
+
+	switch (proto->fam->sock_domain) {
+	case AF_INET: {
+		struct sockaddr_in address = {
+			.sin_family = AF_INET,
+			.sin_port = 0,
+			.sin_addr = { .s_addr = INADDR_ANY }
+		};
+		if (bind(fd, (struct sockaddr *)&address, sizeof(address)) < 0) {
+			send_log(NULL, LOG_WARNING,
+				 "DNS : section '%s': can't bind socket for nameserver '%s' on 0.0.0.0:0.\n",
+				 ns->counters->pid, ns->id);
+			close(fd);
+			return -1;
+		}
+		break;
+	}
+	case AF_INET6: {
+		struct sockaddr_in6 address6 = {
+			.sin6_family = AF_INET6,
+			.sin6_port = 0,
+			.sin6_addr = in6addr_any,
+			.sin6_flowinfo = 0,
+			.sin6_scope_id = 0
+		};
+		if (bind(fd, (struct sockaddr *)&address6, sizeof(address6)) < 0) {
+			send_log(NULL, LOG_WARNING,
+				 "DNS : section '%s': can't bind socket for nameserver '%s' on :::0.\n",
+				 ns->counters->pid, ns->id);
+			close(fd);
+			return -1;
+		}
+		break;
+	}
+	case AF_UNIX:
+		/* if IPC is used via local domain sockets, we don't expect that
+		 * the path to DNS server socket can change dynamically.
+		 */
+		if (connect(fd, (struct sockaddr*)&dgram->addr.to, get_addr_len(&dgram->addr.to)) == -1) {
+			send_log(NULL, LOG_WARNING,
+				 "DNS : section '%s': can't connect socket for nameserver '%s'.\n",
+				 ns->counters->id, ns->id);
+			close(fd);
+			return -1;
+		}
+		break;
+	default:
 		close(fd);
-		return -1;
+		BUG_ON(1, "DNS: Unsupported address family.");
 	}
 
 	/* Make the socket non blocking */
@@ -106,7 +149,17 @@ int dns_send_nameserver(struct dns_nameserver *ns, void *buf, size_t len)
 			fd = dgram->t.sock.fd;
 		}
 
-		ret = send(fd, buf, len, 0);
+		if (dgram->addr.to.ss_family == AF_UNIX) {
+			/* we do connect for AF_UNIX sockets and from the man
+			 * sendto: "If sendto() is  used on a connection-mode
+			 * (SOCK_STREAM, SOCK_SEQPACKET) socket, the arguments
+			 * dest_addr and addrlen are ignored (and the error
+			 * EISCONN may be returned when they are not NULL and 0)..."
+			 */
+			ret = send(fd, buf, len, 0);
+		} else
+			ret = sendto(fd, buf, len, 0, (struct sockaddr*)&dgram->addr.to, get_addr_len(&dgram->addr.to));
+
 		if (ret < 0) {
 			if (errno == EAGAIN || errno == EWOULDBLOCK) {
 				struct ist myist;
@@ -578,6 +631,7 @@ static void dns_session_io_handler(struct appctx *appctx)
 				LIST_APPEND(&ds->queries, &query->list);
 				eb32_insert(&ds->query_ids, &query->qid);
 				ds->onfly_queries++;
+				HA_ATOMIC_STORE(&ds->dss->consecutive_errors, 0);
 			}
 
 			/* update the tx_offset to handle output in 16k streams */
@@ -806,7 +860,7 @@ void dns_session_free(struct dns_session *ds)
 	pool_free(dns_session_pool, ds);
 }
 
-static struct appctx *dns_session_create(struct dns_session *ds);
+static struct appctx *dns_session_create(struct dns_session *ds, int tempo);
 
 static int dns_session_init(struct appctx *appctx)
 {
@@ -831,6 +885,7 @@ static int dns_session_init(struct appctx *appctx)
 
 	applet_expect_no_data(appctx);
 	ds->appctx = appctx;
+	appctx->t->expire = TICK_ETERNITY;
 	return 0;
 
   error:
@@ -844,6 +899,7 @@ static void dns_session_release(struct appctx *appctx)
 {
 	struct dns_session *ds = appctx->svcctx;
 	struct dns_stream_server *dss __maybe_unused;
+	int consecutive_errors;
 
 	if (!ds)
 		return;
@@ -906,13 +962,24 @@ static void dns_session_release(struct appctx *appctx)
 	/* reset offset to be sure to start from message start */
 	ds->tx_msg_offset = 0;
 
+	consecutive_errors = HA_ATOMIC_LOAD(&ds->dss->consecutive_errors);
+	/* we know ds encountered an error because it failed to send all
+	 * its queries: increase consecutive_errors (we take some precautions
+	 * to prevent the counter from overflowing since it is atomically
+	 * updated)
+	 */
+	while (consecutive_errors < DNS_MAX_DSS_CONSECUTIVE_ERRORS &&
+	       !HA_ATOMIC_CAS(&ds->dss->consecutive_errors,
+	                      &consecutive_errors, consecutive_errors + 1) &&
+	       __ha_cpu_relax());
+
 	/* here the ofs and the attached counter
 	 * are kept unchanged
 	 */
 
 	/* Create a new appctx, We hope we can
 	 * create from the release callback! */
-	ds->appctx = dns_session_create(ds);
+	ds->appctx = dns_session_create(ds, 1);
 	if (!ds->appctx) {
 		dns_session_free(ds);
 		HA_SPIN_UNLOCK(DNS_LOCK, &dss->lock);
@@ -937,8 +1004,10 @@ static struct applet dns_session_applet = {
 /*
  * Function used to create an appctx for a DNS session
  * It sets its context into appctx->svcctx.
+ * if <tempo> is set, then the session startup will be delayed by 1
+ * second
  */
-static struct appctx *dns_session_create(struct dns_session *ds)
+static struct appctx *dns_session_create(struct dns_session *ds, int tempo)
 {
 	struct appctx *appctx;
 
@@ -947,10 +1016,14 @@ static struct appctx *dns_session_create(struct dns_session *ds)
 		goto out_close;
 	appctx->svcctx = (void *)ds;
 
-	if (appctx_init(appctx) == -1) {
-		ha_alert("out of memory in dns_session_create().\n");
-		goto out_free_appctx;
+	if (!tempo) {
+		if (appctx_init(appctx) == -1) {
+			ha_alert("out of memory in dns_session_create().\n");
+			goto out_free_appctx;
+		}
 	}
+	else
+		appctx_schedule(appctx, tick_add(now_ms, MS_TO_TICKS(1000)));
 
 	return appctx;
 
@@ -1037,6 +1110,9 @@ struct dns_session *dns_session_new(struct dns_stream_server *dss)
 	if (dss->maxconn && (dss->maxconn <= dss->cur_conns))
 		return NULL;
 
+	if (HA_ATOMIC_LOAD(&dss->consecutive_errors) >= DNS_MAX_DSS_CONSECUTIVE_ERRORS)
+		return NULL;
+
 	ds = pool_zalloc(dns_session_pool);
 	if (!ds)
 		return NULL;
@@ -1072,7 +1148,7 @@ struct dns_session *dns_session_new(struct dns_stream_server *dss)
 	ds->task_exp->process = dns_process_query_exp;
 	ds->task_exp->context = ds;
 
-	ds->appctx = dns_session_create(ds);
+	ds->appctx = dns_session_create(ds, 0);
 	if (!ds->appctx)
 		goto error;
 
