@@ -71,14 +71,13 @@ static void _event_hdl_sub_list_destroy(event_hdl_sub_list *sub_list);
 static void event_hdl_deinit(struct sig_handler *sh)
 {
 	event_hdl_sub_list *cur_list;
-	struct mt_list *elt1, elt2;
+	struct mt_list back;
 
 	/* destroy all known subscription lists */
-	mt_list_for_each_entry_safe(cur_list, &known_event_hdl_sub_list, known, elt1, elt2) {
-		/* remove cur elem from list */
-		MT_LIST_DELETE_SAFE(elt1);
-		/* then destroy it */
+	MT_LIST_FOR_EACH_ENTRY_UNLOCKED(cur_list, &known_event_hdl_sub_list, known, back) {
+		/* remove cur elem from list and free it */
 		_event_hdl_sub_list_destroy(cur_list);
+		cur_list = NULL;
 	}
 }
 
@@ -299,11 +298,11 @@ static inline struct event_hdl_sub_type _event_hdl_getsub_async(struct event_hdl
 	struct mt_list lock;
 	struct event_hdl_sub_type type = EVENT_HDL_SUB_NONE;
 
-	lock = MT_LIST_LOCK_ELT(&cur_sub->mt_list);
+	lock = mt_list_lock_full(&cur_sub->mt_list);
 	if (lock.next != &cur_sub->mt_list)
 		type = _event_hdl_getsub(cur_sub);
 	// else already removed
-	MT_LIST_UNLOCK_ELT(&cur_sub->mt_list, lock);
+	mt_list_unlock_full(&cur_sub->mt_list, lock);
 	return type;
 }
 
@@ -320,11 +319,11 @@ static inline int _event_hdl_resub_async(struct event_hdl_sub *cur_sub, struct e
 	int status = 0;
 	struct mt_list lock;
 
-	lock = MT_LIST_LOCK_ELT(&cur_sub->mt_list);
+	lock = mt_list_lock_full(&cur_sub->mt_list);
 	if (lock.next != &cur_sub->mt_list)
 		status = _event_hdl_resub(cur_sub, type);
 	// else already removed
-	MT_LIST_UNLOCK_ELT(&cur_sub->mt_list, lock);
+	mt_list_unlock_full(&cur_sub->mt_list, lock);
 	return status;
 }
 
@@ -343,13 +342,14 @@ static inline void _event_hdl_unsubscribe(struct event_hdl_sub *del_sub)
 		 * kill themselves (ie: normal async mode) when they receive such event
 		 */
 		HA_ATOMIC_INC(&del_sub->hdl.async_equeue->size);
-		lock = MT_LIST_APPEND_LOCKED(&del_sub->hdl.async_equeue->head, &del_sub->async_end->mt_list);
+		mt_list_lock_elem(&del_sub->async_end->mt_list);
+		lock = mt_list_lock_prev(&del_sub->hdl.async_equeue->head);
 
 		/* wake up the task */
 		event_hdl_task_wakeup(del_sub->hdl.async_task);
 
 		/* unlock END EVENT (we're done, the task is now free to consume it) */
-		MT_LIST_UNLOCK_ELT(&del_sub->async_end->mt_list, lock);
+		mt_list_unlock_full(&del_sub->async_end->mt_list, lock);
 
 		/* we don't free sub here
 		* freeing will be performed by async task so it can safely rely
@@ -419,8 +419,7 @@ static void event_hdl_unsubscribe_sync(const struct event_hdl_sub_mgmt *mgmt)
 		return; /* already removed from sync ctx */
 
 	/* assuming that publish sync code will notice that mgmt->this is NULL
-	 * and will perform the list removal using MT_LIST_DELETE_SAFE and
-	 * _event_hdl_unsubscribe()
+	 * and will perform the list removal and _event_hdl_unsubscribe()
 	 * while still owning the lock
 	 */
 	((struct event_hdl_sub_mgmt *)mgmt)->this = NULL;
@@ -447,7 +446,7 @@ struct event_hdl_sub *event_hdl_subscribe_ptr(event_hdl_sub_list *sub_list,
                                               struct event_hdl_sub_type e_type, struct event_hdl hdl)
 {
 	struct event_hdl_sub *new_sub = NULL;
-	struct mt_list *elt1, elt2;
+	struct mt_list back;
 	struct event_hdl_async_task_default_ctx	*task_ctx = NULL;
 	struct mt_list lock;
 
@@ -523,14 +522,14 @@ struct event_hdl_sub *event_hdl_subscribe_ptr(event_hdl_sub_list *sub_list,
 	/* ready for registration */
 	MT_LIST_INIT(&new_sub->mt_list);
 
-	lock = MT_LIST_LOCK_ELT(&sub_list->known);
+	lock = mt_list_lock_full(&sub_list->known);
 
 	/* check if such identified hdl is not already registered */
 	if (hdl.id) {
 		struct event_hdl_sub *cur_sub;
 		uint8_t found = 0;
 
-		mt_list_for_each_entry_safe(cur_sub, &sub_list->head, mt_list, elt1, elt2) {
+		MT_LIST_FOR_EACH_ENTRY_LOCKED(cur_sub, &sub_list->head, mt_list, back) {
 			if (hdl.id == cur_sub->hdl.id) {
 				/* we found matching registered hdl */
 				found = 1;
@@ -539,7 +538,7 @@ struct event_hdl_sub *event_hdl_subscribe_ptr(event_hdl_sub_list *sub_list,
 		}
 		if (found) {
 			/* error already registered */
-			MT_LIST_UNLOCK_ELT(&sub_list->known, lock);
+			mt_list_unlock_full(&sub_list->known, lock);
 			event_hdl_report_hdl_state(ha_alert, &hdl, "SUB", "could not subscribe: subscription with this id already exists");
 			goto cleanup;
 		}
@@ -551,7 +550,7 @@ struct event_hdl_sub *event_hdl_subscribe_ptr(event_hdl_sub_list *sub_list,
 		 * it is a memory/IO error since it should not be long before haproxy
 		 * enters the deinit() function anyway
 		 */
-		MT_LIST_UNLOCK_ELT(&sub_list->known, lock);
+		mt_list_unlock_full(&sub_list->known, lock);
 		goto cleanup;
 	}
 
@@ -575,7 +574,7 @@ struct event_hdl_sub *event_hdl_subscribe_ptr(event_hdl_sub_list *sub_list,
 		MT_LIST_APPEND(&sub_list->head, &new_sub->mt_list);
 	}
 
-	MT_LIST_UNLOCK_ELT(&sub_list->known, lock);
+	mt_list_unlock_full(&sub_list->known, lock);
 
 	return new_sub;
 
@@ -629,11 +628,11 @@ void event_hdl_pause(struct event_hdl_sub *cur_sub)
 {
 	struct mt_list lock;
 
-	lock = MT_LIST_LOCK_ELT(&cur_sub->mt_list);
+	lock = mt_list_lock_full(&cur_sub->mt_list);
 	if (lock.next != &cur_sub->mt_list)
 		_event_hdl_pause(cur_sub);
 	// else already removed
-	MT_LIST_UNLOCK_ELT(&cur_sub->mt_list, lock);
+	mt_list_unlock_full(&cur_sub->mt_list, lock);
 }
 
 void _event_hdl_resume(struct event_hdl_sub *cur_sub)
@@ -645,11 +644,11 @@ void event_hdl_resume(struct event_hdl_sub *cur_sub)
 {
 	struct mt_list lock;
 
-	lock = MT_LIST_LOCK_ELT(&cur_sub->mt_list);
+	lock = mt_list_lock_full(&cur_sub->mt_list);
 	if (lock.next != &cur_sub->mt_list)
 		_event_hdl_resume(cur_sub);
 	// else already removed
-	MT_LIST_UNLOCK_ELT(&cur_sub->mt_list, lock);
+	mt_list_unlock_full(&cur_sub->mt_list, lock);
 }
 
 void event_hdl_unsubscribe(struct event_hdl_sub *del_sub)
@@ -678,17 +677,18 @@ int event_hdl_lookup_unsubscribe(event_hdl_sub_list *sub_list,
                                  uint64_t lookup_id)
 {
 	struct event_hdl_sub *del_sub = NULL;
-	struct mt_list *elt1, elt2;
+	struct mt_list back;
 	int found = 0;
 
 	if (!sub_list)
 		sub_list = &global_event_hdl_sub_list; /* fall back to global list */
 
-	mt_list_for_each_entry_safe(del_sub, &sub_list->head, mt_list, elt1, elt2) {
+	MT_LIST_FOR_EACH_ENTRY_LOCKED(del_sub, &sub_list->head, mt_list, back) {
 		if (lookup_id == del_sub->hdl.id) {
 			/* we found matching registered hdl */
-			MT_LIST_DELETE_SAFE(elt1);
+			mt_list_unlock_self(&del_sub->mt_list);
 			_event_hdl_unsubscribe(del_sub);
+			del_sub = NULL;
 			found = 1;
 			break; /* id is unique, stop searching */
 		}
@@ -700,13 +700,13 @@ int event_hdl_lookup_resubscribe(event_hdl_sub_list *sub_list,
                                  uint64_t lookup_id, struct event_hdl_sub_type type)
 {
 	struct event_hdl_sub *cur_sub = NULL;
-	struct mt_list *elt1, elt2;
+	struct mt_list back;
 	int status = 0;
 
 	if (!sub_list)
 		sub_list = &global_event_hdl_sub_list; /* fall back to global list */
 
-	mt_list_for_each_entry_safe(cur_sub, &sub_list->head, mt_list, elt1, elt2) {
+	MT_LIST_FOR_EACH_ENTRY_LOCKED(cur_sub, &sub_list->head, mt_list, back) {
 		if (lookup_id == cur_sub->hdl.id) {
 			/* we found matching registered hdl */
 			status = _event_hdl_resub(cur_sub, type);
@@ -720,13 +720,13 @@ int event_hdl_lookup_pause(event_hdl_sub_list *sub_list,
                            uint64_t lookup_id)
 {
 	struct event_hdl_sub *cur_sub = NULL;
-	struct mt_list *elt1, elt2;
+	struct mt_list back;
 	int found = 0;
 
 	if (!sub_list)
 		sub_list = &global_event_hdl_sub_list; /* fall back to global list */
 
-	mt_list_for_each_entry_safe(cur_sub, &sub_list->head, mt_list, elt1, elt2) {
+	MT_LIST_FOR_EACH_ENTRY_LOCKED(cur_sub, &sub_list->head, mt_list, back) {
 		if (lookup_id == cur_sub->hdl.id) {
 			/* we found matching registered hdl */
 			_event_hdl_pause(cur_sub);
@@ -741,13 +741,13 @@ int event_hdl_lookup_resume(event_hdl_sub_list *sub_list,
                             uint64_t lookup_id)
 {
 	struct event_hdl_sub *cur_sub = NULL;
-	struct mt_list *elt1, elt2;
+	struct mt_list back;
 	int found = 0;
 
 	if (!sub_list)
 		sub_list = &global_event_hdl_sub_list; /* fall back to global list */
 
-	mt_list_for_each_entry_safe(cur_sub, &sub_list->head, mt_list, elt1, elt2) {
+	MT_LIST_FOR_EACH_ENTRY_LOCKED(cur_sub, &sub_list->head, mt_list, back) {
 		if (lookup_id == cur_sub->hdl.id) {
 			/* we found matching registered hdl */
 			_event_hdl_resume(cur_sub);
@@ -762,13 +762,13 @@ struct event_hdl_sub *event_hdl_lookup_take(event_hdl_sub_list *sub_list,
                                             uint64_t lookup_id)
 {
 	struct event_hdl_sub *cur_sub = NULL;
-	struct mt_list *elt1, elt2;
+	struct mt_list back;
 	uint8_t found = 0;
 
 	if (!sub_list)
 		sub_list = &global_event_hdl_sub_list; /* fall back to global list */
 
-	mt_list_for_each_entry_safe(cur_sub, &sub_list->head, mt_list, elt1, elt2) {
+	MT_LIST_FOR_EACH_ENTRY_LOCKED(cur_sub, &sub_list->head, mt_list, back) {
 		if (lookup_id == cur_sub->hdl.id) {
 			/* we found matching registered hdl */
 			event_hdl_take(cur_sub);
@@ -787,11 +787,11 @@ static int _event_hdl_publish(event_hdl_sub_list *sub_list, struct event_hdl_sub
                               const struct event_hdl_cb_data *data)
 {
 	struct event_hdl_sub *cur_sub;
-	struct mt_list *elt1, elt2;
+	struct mt_list back;
 	struct event_hdl_async_event_data *async_data = NULL; /* reuse async data for multiple async hdls */
 	int error = 0;
 
-	mt_list_for_each_entry_safe(cur_sub, &sub_list->head, mt_list, elt1, elt2) {
+	MT_LIST_FOR_EACH_ENTRY_LOCKED(cur_sub, &sub_list->head, mt_list, back) {
 		/* notify each function that has subscribed to sub_family.type, unless paused */
 		if ((cur_sub->sub.family == e_type.family) &&
 		    ((cur_sub->sub.subtype & e_type.subtype) == e_type.subtype) &&
@@ -821,10 +821,11 @@ static int _event_hdl_publish(event_hdl_sub_list *sub_list, struct event_hdl_sub
 				if (!sub_mgmt.this) {
 					/* user has performed hdl unsub
 					 * we must remove it from the list
+					 * then free it.
 					 */
-					MT_LIST_DELETE_SAFE(elt1);
-					/* then free it */
+					mt_list_unlock_self(&cur_sub->mt_list);
 					_event_hdl_unsubscribe(cur_sub);
+					cur_sub = NULL;
 				}
 			} else {
 				/* async mode: here we need to prepare event data
@@ -954,13 +955,12 @@ void event_hdl_sub_list_init(event_hdl_sub_list *sub_list)
 static void _event_hdl_sub_list_destroy(event_hdl_sub_list *sub_list)
 {
 	struct event_hdl_sub *cur_sub;
-	struct mt_list *elt1, elt2;
+	struct mt_list back;
 
-	mt_list_for_each_entry_safe(cur_sub, &sub_list->head, mt_list, elt1, elt2) {
-		/* remove cur elem from list */
-		MT_LIST_DELETE_SAFE(elt1);
-		/* then free it */
+	MT_LIST_FOR_EACH_ENTRY_UNLOCKED(cur_sub, &sub_list->head, mt_list, back) {
+		/* remove cur elem from list and free it */
 		_event_hdl_unsubscribe(cur_sub);
+		cur_sub = NULL;
 	}
 }
 
