@@ -112,6 +112,7 @@ volatile struct fdlist update_list[MAX_TGROUPS]; // Global update list
 
 THREAD_LOCAL int *fd_updt  = NULL;  // FD updates list
 THREAD_LOCAL int  fd_nbupdt = 0;   // number of updates in the list
+THREAD_LOCAL int  fd_highest = -1; // highest FD known by the current thread
 THREAD_LOCAL int poller_rd_pipe = -1; // Pipe to wake the thread
 int poller_wr_pipe[MAX_THREADS] __read_mostly; // Pipe to wake the threads
 
@@ -841,7 +842,7 @@ void fd_reregister_all(int tgrp, ulong mask)
 {
 	int fd;
 
-	for (fd = 0; fd < global.maxsock; fd++) {
+	for (fd = 0; fd < fd_highest; fd++) {
 		if (!fdtab[fd].owner)
 			continue;
 
@@ -1024,32 +1025,6 @@ void my_closefrom(int start)
 }
 #endif // defined(USE_POLL)
 
-/* Sets the RLIMIT_NOFILE setting to <new_limit> and returns the previous one
- * in <old_limit> if the pointer is not NULL, even if set_rlimit() fails. The
- * two pointers may point to the same variable as the copy happens after
- * setting the new value. The value is only changed if at least one of the new
- * limits is strictly higher than the current one, otherwise returns 0 without
- * changing anything. The getrlimit() or setrlimit() syscall return value is
- * returned and errno is preserved.
- */
-int raise_rlim_nofile(struct rlimit *old_limit, struct rlimit *new_limit)
-{
-	struct rlimit limit = { };
-	int ret = 0;
-
-	ret = getrlimit(RLIMIT_NOFILE, &limit);
-
-	if (ret == 0 &&
-	    (limit.rlim_max < new_limit->rlim_max ||
-	     limit.rlim_cur < new_limit->rlim_cur)) {
-		ret = setrlimit(RLIMIT_NOFILE, new_limit);
-	}
-
-	if (old_limit)
-		*old_limit = limit;
-
-	return ret;
-}
 
 /* Computes the bounded poll() timeout based on the next expiration timer <next>
  * by bounding it to MAX_DELAY_MS. <next> may equal TICK_ETERNITY. The pollers
@@ -1302,7 +1277,7 @@ int list_pollers(FILE *out)
 int fork_poller()
 {
 	int fd;
-	for (fd = 0; fd < global.maxsock; fd++) {
+	for (fd = 0; fd < fd_highest; fd++) {
 		if (fdtab[fd].owner) {
 			HA_ATOMIC_OR(&fdtab[fd].state, FD_CLONED);
 		}
