@@ -432,6 +432,7 @@ static void spoe_release_appctx(struct appctx *appctx)
 	/* Shutdown the server connection, if needed */
 	if (appctx->st0 != SPOE_APPCTX_ST_END) {
 		appctx->st0 = SPOE_APPCTX_ST_END;
+		applet_set_error(appctx);
 		if (spoe_appctx->status_code == SPOP_ERR_NONE)
 			spoe_appctx->status_code = SPOP_ERR_IO;
 	}
@@ -502,14 +503,15 @@ static void spoe_handle_appctx(struct appctx *appctx)
 		goto out;
 	}
 
-	if (!SPOE_APPCTX(appctx)->spoe_ctx)
-		appctx->st0 =  SPOE_APPCTX_ST_EXIT;
-
   switchstate:
 	switch (appctx->st0) {
 		/* case SPOE_APPCTX_ST_PROCESSING: */
 		case SPOE_APPCTX_ST_WAITING_ACK:
-			if (!spoe_handle_receiving_frame_appctx(appctx))
+			if (!SPOE_APPCTX(appctx)->spoe_ctx) {
+				appctx->st0 = SPOE_APPCTX_ST_END;
+				applet_set_error(appctx);
+			}
+			else if (!spoe_handle_receiving_frame_appctx(appctx))
 				break;
 			goto switchstate;
 
@@ -652,13 +654,14 @@ static int spoe_encode_messages(struct stream *s, struct spoe_context *ctx,
 	struct spoe_config  *conf = FLT_CONF(ctx->filter);
 	struct spoe_agent   *agent = conf->agent;
 	struct spoe_message *msg;
-	char   *p, *end;
+	char   *p, *start, *end;
 
 	p   = b_head(&ctx->buffer);
 	end =  p + agent->max_frame_size - SPOP_FRAME_HDR_SIZE;
 
 	/* Set Frame type */
 	*p++ = SPOP_FRM_T_HAPROXY_NOTIFY;
+	start = p;
 
 	if (type == SPOE_MSGS_BY_EVENT) { /* Loop on messages by event */
 		list_for_each_entry(msg, messages, by_evt) {
@@ -678,7 +681,7 @@ static int spoe_encode_messages(struct stream *s, struct spoe_context *ctx,
 
 
 	/* nothing has been encoded */
-	if (p == b_head(&ctx->buffer))
+	if (p == start)
 		goto skip;
 
 	b_set_data(&ctx->buffer, p - b_head(&ctx->buffer));
@@ -1188,6 +1191,10 @@ static void spoe_destroy_context(struct filter *filter)
 	if (!ctx)
 		return;
 
+	if (ctx->state != SPOE_CTX_ST_NONE || ctx->state == SPOE_CTX_ST_READY) {
+		ctx->status_code = SPOE_CTX_ERR_INTERRUPT;
+		_HA_ATOMIC_INC(&conf->agent->counters.nb_errors);
+	}
 	spoe_stop_processing(conf->agent, ctx);
 	pool_free(pool_head_spoe_ctx, ctx);
 	filter->ctx = NULL;
