@@ -148,9 +148,10 @@ struct h3c {
 
 DECLARE_STATIC_POOL(pool_head_h3c, "h3c", sizeof(struct h3c));
 
-#define H3_SF_UNI_INIT  0x00000001  /* stream type not parsed for unidirectional stream */
-#define H3_SF_UNI_NO_H3 0x00000002  /* unidirectional stream does not carry H3 frames */
-#define H3_SF_HAVE_CLEN 0x00000004  /* content-length header is present */
+#define H3_SF_UNI_INIT     0x00000001  /* stream type not parsed for unidirectional stream */
+#define H3_SF_UNI_NO_H3    0x00000002  /* unidirectional stream does not carry H3 frames */
+#define H3_SF_HAVE_CLEN    0x00000004  /* content-length header is present */
+#define H3_SF_SENT_INTERIM 0x00000008  /* last response sent is 1xx interim. Used on FE side only. */
 
 struct h3s {
 	struct h3c *h3c;
@@ -757,6 +758,21 @@ static ssize_t h3_headers_to_htx(struct qcs *qcs, const struct buffer *buf,
 		goto out;
 	}
 
+	/* We're going to concatenate :authority with :path to form a URI. Some
+	 * characters must absolutely be avoided in :authority to make sure not
+	 * to result in a broken concatenation. See the following links for a
+	 * discussion on this topic:
+	 *   https://github.com/httpwg/http2-spec/pull/936
+	 *   https://github.com/haproxy/haproxy/issues/2941
+	 */
+	if (http_authority_has_forbidden_char(authority)) {
+		TRACE_ERROR("invalid character in authority", H3_EV_RX_FRAME|H3_EV_RX_HDR, qcs->qcc->conn, qcs);
+		h3s->err = H3_ERR_MESSAGE_ERROR;
+		qcc_report_glitch(h3c->qcc, 1);
+		len = -1;
+		goto out;
+	}
+
 	/* Ensure that final URI does not contains LWS nor CTL characters. */
 	for (i = 0; i < path.len; i++) {
 		unsigned char c = istptr(path)[i];
@@ -802,7 +818,7 @@ static ssize_t h3_headers_to_htx(struct qcs *qcs, const struct buffer *buf,
 
 		for (i = 0; i < list[hdr_idx].n.len; ++i) {
 			const char c = list[hdr_idx].n.ptr[i];
-			if ((uint8_t)(c - 'A') < 'Z' - 'A' || !HTTP_IS_TOKEN(c)) {
+			if ((uint8_t)(c - 'A') <= 'Z' - 'A' || !HTTP_IS_TOKEN(c)) {
 				TRACE_ERROR("invalid characters in field name", H3_EV_RX_FRAME|H3_EV_RX_HDR, qcs->qcc->conn, qcs);
 				h3s->err = H3_ERR_MESSAGE_ERROR;
 				qcc_report_glitch(h3c->qcc, 1);
@@ -835,7 +851,8 @@ static ssize_t h3_headers_to_htx(struct qcs *qcs, const struct buffer *buf,
 		if (isteq(list[hdr_idx].n, ist("host"))) {
 			struct ist prev_auth = authority;
 
-			if (h3_set_authority(qcs, &authority, list[hdr_idx].v)) {
+			if (http_authority_has_forbidden_char(list[hdr_idx].v) ||
+			    h3_set_authority(qcs, &authority, list[hdr_idx].v)) {
 				h3s->err = H3_ERR_MESSAGE_ERROR;
 				qcc_report_glitch(h3c->qcc, 1);
 				len = -1;
@@ -1088,7 +1105,7 @@ static ssize_t h3_trailers_to_htx(struct qcs *qcs, const struct buffer *buf,
 
 		for (i = 0; i < list[hdr_idx].n.len; ++i) {
 			const char c = list[hdr_idx].n.ptr[i];
-			if ((uint8_t)(c - 'A') < 'Z' - 'A' || !HTTP_IS_TOKEN(c)) {
+			if ((uint8_t)(c - 'A') <= 'Z' - 'A' || !HTTP_IS_TOKEN(c)) {
 				TRACE_ERROR("invalid characters in field name", H3_EV_RX_FRAME|H3_EV_RX_HDR, qcs->qcc->conn, qcs);
 				h3s->err = H3_ERR_MESSAGE_ERROR;
 				qcc_report_glitch(h3c->qcc, 1);
@@ -1654,6 +1671,7 @@ static int h3_encode_header(struct buffer *buf,
 
 static int h3_resp_headers_send(struct qcs *qcs, struct htx *htx)
 {
+	struct h3s *h3s = qcs->ctx;
 	int err;
 	struct buffer outbuf;
 	struct buffer headers_buf = BUF_NULL;
@@ -1685,8 +1703,20 @@ static int h3_resp_headers_send(struct qcs *qcs, struct htx *htx)
 			/* start-line -> HEADERS h3 frame */
 			BUG_ON(sl);
 			sl = htx_get_blk_ptr(htx, blk);
-			/* TODO should be on h3 layer */
 			status = sl->info.res.status;
+			if (status < 100 || status > 999) {
+				TRACE_ERROR("invalid response status code", H3_EV_STRM_SEND, qcs->qcc->conn, qcs);
+				goto err;
+			}
+			else if (status >= 100 && status < 200) {
+				TRACE_USER("handling interim HTX response", H3_EV_STRM_SEND, qcs->qcc->conn, qcs);
+				BUG_ON(conn_is_back(qcs->qcc->conn)); /* H3_SF_SENT_INTERIM is FE side only. */
+				h3s->flags |= H3_SF_SENT_INTERIM;
+			}
+			else {
+				TRACE_USER("handling final HTX response", H3_EV_STRM_SEND, qcs->qcc->conn, qcs);
+				h3s->flags &= ~H3_SF_SENT_INTERIM;
+			}
 		}
 		else if (type == HTX_BLK_HDR) {
 			if (unlikely(hdr >= sizeof(list) / sizeof(list[0]) - 1)) {
@@ -1704,6 +1734,9 @@ static int h3_resp_headers_send(struct qcs *qcs, struct htx *htx)
 		}
 	}
 
+	/* Current function expects HTX start-line to be present. This also
+	 * ensures <status> conformance has been checked prior to encoding it.
+	 */
 	BUG_ON(!sl);
 
 	list[hdr].n = ist("");
@@ -1721,20 +1754,26 @@ static int h3_resp_headers_send(struct qcs *qcs, struct htx *htx)
 		goto end;
 	}
 
-	/* Buffer allocated just now : must be enough for frame type + length as a max varint size */
-	BUG_ON(b_room(res) < 5);
+	/* Reserve space for frame type + length as a max varint size. */
+	if (unlikely(b_contig_space(res) < 5)) {
+		/* Most of the times, h3_resp_headers_send() is only called one
+		 * time per stream, so buffer will be empty. However, this
+		 * assumption is invalid when handling interim responses, so
+		 * it's important to check out buffer remaining space.
+		 */
+		goto err_full;
+	}
 
 	b_reset(&outbuf);
 	outbuf = b_make(b_tail(res), b_contig_space(res), 0, 0);
 	/* Start the headers after frame type + length */
-	headers_buf = b_make(b_head(res) + 5, b_size(res) - 5, 0, 0);
+	headers_buf = b_make(b_tail(res) + 5, b_contig_space(res) - 5, 0, 0);
 
 	TRACE_DATA("encoding HEADERS frame", H3_EV_TX_FRAME|H3_EV_TX_HDR,
 	           qcs->qcc->conn, qcs);
 	if (qpack_encode_field_section_line(&headers_buf))
 		goto err_full;
 	if (qpack_encode_int_status(&headers_buf, status)) {
-		/* TODO handle invalid status code VS no buf space left */
 		TRACE_ERROR("error during status code encoding", H3_EV_TX_FRAME|H3_EV_TX_HDR, qcs->qcc->conn, qcs);
 		goto err_full;
 	}
@@ -1795,6 +1834,20 @@ static int h3_resp_headers_send(struct qcs *qcs, struct htx *htx)
 	return ret;
 
  err_full:
+	if (b_data(res)) {
+		/* Output buffer already contains data : this may happens after
+		 * HTTP interim response encoding. Try to release buffer to be
+		 * able to encode newer interim or final response.
+		 */
+		if (qcc_release_stream_txbuf(qcs)) {
+			TRACE_DEVEL("cannot release buf", H3_EV_TX_FRAME|H3_EV_TX_HDR, qcs->qcc->conn, qcs);
+			return 0;
+		}
+
+		TRACE_DEVEL("retry after buf release", H3_EV_TX_FRAME|H3_EV_TX_HDR, qcs->qcc->conn, qcs);
+		goto retry;
+	}
+
 	if (smallbuf) {
 		TRACE_DEVEL("retry with a full buffer", H3_EV_TX_FRAME|H3_EV_TX_HDR, qcs->qcc->conn, qcs);
 		smallbuf = 0;
@@ -2107,19 +2160,24 @@ static int h3_resp_data_send(struct qcs *qcs, struct htx *htx,
 	return -1;
 }
 
-static size_t h3_snd_buf(struct qcs *qcs, struct buffer *buf, size_t count)
+static size_t h3_snd_buf(struct qcs *qcs, struct buffer *buf, size_t count, char *fin)
 {
+	struct h3s *h3s = qcs->ctx;
 	size_t total = 0;
 	enum htx_blk_type btype;
 	struct htx *htx;
 	struct htx_blk *blk;
 	uint32_t bsize;
 	int32_t idx;
+	char eom;
 	int ret = 0;
 
 	TRACE_ENTER(H3_EV_STRM_SEND, qcs->qcc->conn, qcs);
 
+	*fin = 0;
 	htx = htx_from_buf(buf);
+	/* EOM is saved here, useful if 0-copy is performed with HTX buf. */
+	eom = htx->flags & HTX_FL_EOM;
 
 	while (count && !htx_is_empty(htx) && qcc_stream_can_send(qcs) && ret >= 0) {
 		idx = htx_get_head(htx);
@@ -2132,7 +2190,7 @@ static size_t h3_snd_buf(struct qcs *qcs, struct buffer *buf, size_t count)
 
 		switch (btype) {
 		case HTX_BLK_RES_SL:
-			/* start-line -> HEADERS h3 frame */
+			/* start-line -> HEADERS h3 frame (FE side) */
 			ret = h3_resp_headers_send(qcs, htx);
 			if (ret > 0) {
 				total += ret;
@@ -2213,6 +2271,10 @@ static size_t h3_snd_buf(struct qcs *qcs, struct buffer *buf, size_t count)
 #endif
 
  out:
+	if (eom && htx_is_empty(htx) && !(h3s->flags & H3_SF_SENT_INTERIM)) {
+		TRACE_USER("transcoding last HTX message", H3_EV_STRM_SEND, qcs->qcc->conn, qcs);
+		*fin = 1;
+	}
 	htx_to_buf(htx, buf);
 
 	TRACE_LEAVE(H3_EV_STRM_SEND, qcs->qcc->conn, qcs);
@@ -2512,6 +2574,21 @@ static int h3_finalize(void *ctx)
 
 	TRACE_ENTER(H3_EV_H3C_NEW, qcc->conn);
 
+	/* RFC 9114 6.2. Unidirectional Streams
+	 *
+	 * Each endpoint needs to create at least one unidirectional stream for
+	 * the HTTP control stream. QPACK requires two additional unidirectional
+	 * streams, and other extensions might require further streams.
+	 * Therefore, the transport parameters sent by both clients and servers
+	 * MUST allow the peer to create at least three unidirectional streams.
+	 */
+	if (qcc_fctl_avail_streams(qcc, 0) < 3) {
+		TRACE_ERROR("peer flow-control limit does not allow control stream creation", H3_EV_H3C_NEW, qcc->conn);
+		qcc_set_error(qcc, H3_ERR_GENERAL_PROTOCOL_ERROR, 1);
+		qcc_report_glitch(qcc, 1);
+		goto err;
+	}
+
 	qcs = qcc_init_stream_local(qcc, 0);
 	if (!qcs) {
 		/* Error must be set by qcc_init_stream_local(). */
