@@ -10,6 +10,11 @@
  *
  */
 
+/* this is to have tcp_info defined on systems using musl
+ * library, such as Alpine Linux.
+ */
+#define _GNU_SOURCE
+
 #include <ctype.h>
 #include <errno.h>
 #include <stdio.h>
@@ -48,6 +53,7 @@ static int tcp_suspend_receiver(struct receiver *rx);
 static int tcp_resume_receiver(struct receiver *rx);
 static void tcp_enable_listener(struct listener *listener);
 static void tcp_disable_listener(struct listener *listener);
+static int tcp_get_info(struct connection *conn, long long int *info, int info_num);
 
 /* Note: must not be declared <const> as its list will be overwritten */
 struct protocol proto_tcpv4 = {
@@ -69,6 +75,7 @@ struct protocol proto_tcpv4 = {
 	.drain          = sock_drain,
 	.check_events   = sock_check_events,
 	.ignore_events  = sock_ignore_events,
+	.get_info       = tcp_get_info,
 
 	/* binding layer */
 	.rx_suspend     = tcp_suspend_receiver,
@@ -86,8 +93,6 @@ struct protocol proto_tcpv4 = {
 	.rx_unbind      = sock_unbind,
 	.rx_listening   = sock_accepting_conn,
 	.default_iocb   = sock_accept_iocb,
-	.receivers      = LIST_HEAD_INIT(proto_tcpv4.receivers),
-	.nb_receivers   = 0,
 #ifdef SO_REUSEPORT
 	.flags          = PROTO_F_REUSEPORT_SUPPORTED,
 #endif
@@ -115,6 +120,7 @@ struct protocol proto_tcpv6 = {
 	.drain          = sock_drain,
 	.check_events   = sock_check_events,
 	.ignore_events  = sock_ignore_events,
+	.get_info       = tcp_get_info,
 
 	/* binding layer */
 	.rx_suspend     = tcp_suspend_receiver,
@@ -132,8 +138,6 @@ struct protocol proto_tcpv6 = {
 	.rx_unbind      = sock_unbind,
 	.rx_listening   = sock_accepting_conn,
 	.default_iocb   = sock_accept_iocb,
-	.receivers      = LIST_HEAD_INIT(proto_tcpv6.receivers),
-	.nb_receivers   = 0,
 #ifdef SO_REUSEPORT
 	.flags          = PROTO_F_REUSEPORT_SUPPORTED,
 #endif
@@ -141,6 +145,98 @@ struct protocol proto_tcpv6 = {
 
 INITCALL1(STG_REGISTER, protocol_register, &proto_tcpv6);
 
+#ifdef HA_HAVE_MPTCP
+/* Most fields are copied from proto_tcpv4 */
+struct protocol proto_mptcpv4 = {
+	.name           = "mptcpv4",
+
+	/* connection layer */
+	.xprt_type      = PROTO_TYPE_STREAM,
+	.listen         = tcp_bind_listener,
+	.enable         = tcp_enable_listener,
+	.disable        = tcp_disable_listener,
+	.add            = default_add_listener,
+	.unbind         = default_unbind_listener,
+	.suspend        = default_suspend_listener,
+	.resume         = default_resume_listener,
+	.accept_conn    = sock_accept_conn,
+	.ctrl_init      = sock_conn_ctrl_init,
+	.ctrl_close     = sock_conn_ctrl_close,
+	.connect        = tcp_connect_server,
+	.drain          = sock_drain,
+	.check_events   = sock_check_events,
+	.ignore_events  = sock_ignore_events,
+	.get_info       = tcp_get_info,
+
+	/* binding layer */
+	.rx_suspend     = tcp_suspend_receiver,
+	.rx_resume      = tcp_resume_receiver,
+
+	/* address family */
+	.fam            = &proto_fam_inet4,
+
+	/* socket layer */
+	.proto_type     = PROTO_TYPE_STREAM,
+	.sock_type      = SOCK_STREAM,
+	.sock_prot      = IPPROTO_MPTCP,		/* MPTCP specific */
+	.rx_enable      = sock_enable,
+	.rx_disable     = sock_disable,
+	.rx_unbind      = sock_unbind,
+	.rx_listening   = sock_accepting_conn,
+	.default_iocb   = sock_accept_iocb,
+#ifdef SO_REUSEPORT
+	.flags          = PROTO_F_REUSEPORT_SUPPORTED,
+#endif
+};
+
+INITCALL1(STG_REGISTER, protocol_register, &proto_mptcpv4);
+
+/* Most fields are copied from proto_tcpv6 */
+struct protocol proto_mptcpv6 = {
+	.name           = "mptcpv6",
+
+	/* connection layer */
+	.xprt_type      = PROTO_TYPE_STREAM,
+	.listen         = tcp_bind_listener,
+	.enable         = tcp_enable_listener,
+	.disable        = tcp_disable_listener,
+	.add            = default_add_listener,
+	.unbind         = default_unbind_listener,
+	.suspend        = default_suspend_listener,
+	.resume         = default_resume_listener,
+	.accept_conn    = sock_accept_conn,
+	.ctrl_init      = sock_conn_ctrl_init,
+	.ctrl_close     = sock_conn_ctrl_close,
+	.connect        = tcp_connect_server,
+	.drain          = sock_drain,
+	.check_events   = sock_check_events,
+	.ignore_events  = sock_ignore_events,
+	.get_info       = tcp_get_info,
+
+	/* binding layer */
+	.rx_suspend     = tcp_suspend_receiver,
+	.rx_resume      = tcp_resume_receiver,
+
+	/* address family */
+	.fam            = &proto_fam_inet6,
+
+	/* socket layer */
+	.proto_type     = PROTO_TYPE_STREAM,
+	.sock_type      = SOCK_STREAM,
+	.sock_prot      = IPPROTO_MPTCP,		/* MPTCP specific */
+	.rx_enable      = sock_enable,
+	.rx_disable     = sock_disable,
+	.rx_unbind      = sock_unbind,
+	.rx_listening   = sock_accepting_conn,
+	.default_iocb   = sock_accept_iocb,
+#ifdef SO_REUSEPORT
+	.flags          = PROTO_F_REUSEPORT_SUPPORTED,
+#endif
+};
+
+INITCALL1(STG_REGISTER, protocol_register, &proto_mptcpv6);
+#endif
+
 /* Binds ipv4/ipv6 address <local> to socket <fd>, unless <flags> is set, in which
  * case we try to bind <remote>. <flags> is a 2-bit field consisting of :
  *  - 0 : ignore remote address (may even be a NULL pointer)
@@ -578,25 +674,35 @@ int tcp_bind_listener(struct listener *listener, char *errmsg, int errlen)
 	if (listener->bind_conf->maxseg > 0) {
 		if (setsockopt(fd, IPPROTO_TCP, TCP_MAXSEG,
 			       &listener->bind_conf->maxseg, sizeof(listener->bind_conf->maxseg)) == -1) {
-			chunk_appendf(msg, "%scannot set MSS to %d", msg->data ? ", " : "", listener->bind_conf->maxseg);
+			chunk_appendf(msg, "%scannot set MSS to %d, (%s)", msg->data ? ", " : "", listener->bind_conf->maxseg,
+				      strerror(errno));
 			err |= ERR_WARN;
 		}
 	} else {
 		/* we may want to try to restore the default MSS if the socket was inherited */
 		int tmpmaxseg = -1;
 		int defaultmss;
+		int v4 = listener->rx.addr.ss_family == AF_INET;
 		socklen_t len = sizeof(tmpmaxseg);
 
-		if (listener->rx.addr.ss_family == AF_INET)
-			defaultmss = sock_inet_tcp_maxseg_default;
-		else
-			defaultmss = sock_inet6_tcp_maxseg_default;
+		if (listener->rx.proto->sock_prot == IPPROTO_MPTCP) {
+			if (v4)
+				defaultmss = sock_inet_mptcp_maxseg_default;
+			else
+				defaultmss = sock_inet6_mptcp_maxseg_default;
+		} else {
+			if (v4)
+				defaultmss = sock_inet_tcp_maxseg_default;
+			else
+				defaultmss = sock_inet6_tcp_maxseg_default;
+		}
 
 		getsockopt(fd, IPPROTO_TCP, TCP_MAXSEG, &tmpmaxseg, &len);
 		if (defaultmss > 0 &&
 		    tmpmaxseg != defaultmss &&
 		    setsockopt(fd, IPPROTO_TCP, TCP_MAXSEG, &defaultmss, sizeof(defaultmss)) == -1) {
-			chunk_appendf(msg, "%scannot set MSS to %d", msg->data ? ", " : "", defaultmss);
+			chunk_appendf(msg, "%scannot set MSS to %d, (%s)", msg->data ? ", " : "", defaultmss,
+				      strerror(errno));
 			err |= ERR_WARN;
 		}
 	}
@@ -605,7 +711,8 @@ int tcp_bind_listener(struct listener *listener, char *errmsg, int errlen)
 	if (listener->bind_conf->tcp_ut) {
 		if (setsockopt(fd, IPPROTO_TCP, TCP_USER_TIMEOUT,
 			       &listener->bind_conf->tcp_ut, sizeof(listener->bind_conf->tcp_ut)) == -1) {
-			chunk_appendf(msg, "%scannot set TCP User Timeout", msg->data ? ", " : "");
+			chunk_appendf(msg, "%scannot set TCP User Timeout, (%s)", msg->data ? ", " : "",
+				      strerror(errno));
 			err |= ERR_WARN;
 		}
 	} else
@@ -617,7 +724,8 @@ int tcp_bind_listener(struct listener *listener, char *errmsg, int errlen)
 		/* defer accept by up to one second */
 		int accept_delay = 1;
 		if (setsockopt(fd, IPPROTO_TCP, TCP_DEFER_ACCEPT, &accept_delay, sizeof(accept_delay)) == -1) {
-			chunk_appendf(msg, "%scannot enable DEFER_ACCEPT", msg->data ? ", " : "");
+			chunk_appendf(msg, "%scannot enable DEFER_ACCEPT, (%s)", msg->data ? ", " : "",
+				      strerror(errno));
 			err |= ERR_WARN;
 		}
 	} else
@@ -629,7 +737,8 @@ int tcp_bind_listener(struct listener *listener, char *errmsg, int errlen)
 		/* TFO needs a queue length, let's use the configured backlog */
 		int qlen = listener_backlog(listener);
 		if (setsockopt(fd, IPPROTO_TCP, TCP_FASTOPEN, &qlen, sizeof(qlen)) == -1) {
-			chunk_appendf(msg, "%scannot enable TCP_FASTOPEN", msg->data ? ", " : "");
+			chunk_appendf(msg, "%scannot enable TCP_FASTOPEN, (%s)", msg->data ? ", " : "",
+				      strerror(errno));
 			err |= ERR_WARN;
 		}
 	} else {
@@ -643,7 +752,8 @@ int tcp_bind_listener(struct listener *listener, char *errmsg, int errlen)
 		    qlen != 0) {
 			if (setsockopt(fd, IPPROTO_TCP, TCP_FASTOPEN, &zero,
 			    sizeof(zero)) == -1) {
-				chunk_appendf(msg, "%scannot disable TCP_FASTOPEN", msg->data ? ", " : "");
+				chunk_appendf(msg, "%scannot disable TCP_FASTOPEN, (%s)", msg->data ? ", " : "",
+					      strerror(errno));
 				err |= ERR_WARN;
 			}
 		}
@@ -655,7 +765,8 @@ int tcp_bind_listener(struct listener *listener, char *errmsg, int errlen)
 	if (!ready && /* only listen if not already done by external process */
 	    listen(fd, listener_backlog(listener)) == -1) {
 		err |= ERR_RETRYABLE | ERR_ALERT;
-		chunk_appendf(msg, "%scannot listen to socket", msg->data ? ", " : "");
+		chunk_appendf(msg, "%scannot listen to socket: (%s)", msg->data ? ", " : "",
+			      strerror(errno));
 		goto tcp_close_return;
 	}
 
@@ -666,7 +777,8 @@ int tcp_bind_listener(struct listener *listener, char *errmsg, int errlen)
 		memset(&accept, 0, sizeof(accept));
 		strlcpy2(accept.af_name, "dataready", sizeof(accept.af_name));
 		if (setsockopt(fd, SOL_SOCKET, SO_ACCEPTFILTER, &accept, sizeof(accept)) == -1) {
-			chunk_appendf(msg, "%scannot enable ACCEPT_FILTER", msg->data ? ", " : "");
+			chunk_appendf(msg, "%scannot enable ACCEPT_FILTER, (%s)", msg->data ? ", " : "",
+				      strerror(errno));
 			err |= ERR_WARN;
 		}
 	}
@@ -726,7 +838,7 @@ static int tcp_suspend_receiver(struct receiver *rx)
 	 * parent process and any possible subsequent worker inheriting it.
 	 * Thus we just stop receiving from it.
 	 */
-	if (rx->flags & RX_F_INHERITED_SOCK)
+	if (rx->flags & RX_F_INHERITED)
 		goto done;
 
 	if (connect(rx->fd, &sa, sizeof(sa)) < 0)
@@ -762,13 +874,71 @@ static int tcp_resume_receiver(struct receiver *rx)
 	if (rx->fd < 0)
 		return 0;
 
-	if ((rx->flags & RX_F_INHERITED_SOCK) || listen(rx->fd, listener_backlog(l)) == 0) {
+	if ((rx->flags & RX_F_INHERITED) || listen(rx->fd, listener_backlog(l)) == 0) {
 		fd_want_recv(l->rx.fd);
 		return 1;
 	}
 	return -1;
 }
 
+#ifdef TCP_INFO
+/* Returns some tcp_info data if it's available for <conn> connection into <*info>.
+ * "info_num" represents the required value.
+ * If the function fails it returns 0, otherwise it returns 1 and "result" is filled.
+ */
+static int tcp_get_info(struct connection *conn, long long int *info, int info_num)
+{
+	struct tcp_info tcp_info;
+	socklen_t optlen;
+
+	/* The fd may not be available for the tcp_info struct, and the
+	  syscal can fail. */
+	optlen = sizeof(tcp_info);
+	if ((conn->flags & CO_FL_FDLESS) ||
+	    getsockopt(conn->handle.fd, IPPROTO_TCP, TCP_INFO, &tcp_info, &optlen) == -1)
+		return 0;
+
+	switch (info_num) {
+#if defined(__APPLE__)
+	case 0:  *info = tcp_info.tcpi_rttcur;         break;
+	case 1:  *info = tcp_info.tcpi_rttvar;         break;
+	case 2:  *info = tcp_info.tcpi_tfo_syn_data_acked; break;
+	case 4:  *info = tcp_info.tcpi_tfo_syn_loss;   break;
+	case 5:  *info = tcp_info.tcpi_rto;            break;
+#else
+	/* all other platforms supporting TCP_INFO have these ones */
+	case 0:  *info = tcp_info.tcpi_rtt;            break;
+	case 1:  *info = tcp_info.tcpi_rttvar;         break;
+# if defined(__linux__)
+	/* these ones are common to all Linux versions */
+	case 2:  *info = tcp_info.tcpi_unacked;        break;
+	case 3:  *info = tcp_info.tcpi_sacked;         break;
+	case 4:  *info = tcp_info.tcpi_lost;           break;
+	case 5:  *info = tcp_info.tcpi_retrans;        break;
+	case 6:  *info = tcp_info.tcpi_fackets;        break;
+	case 7:  *info = tcp_info.tcpi_reordering;     break;
+# elif defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__)
+	/* the ones are found on FreeBSD, NetBSD and OpenBSD featuring TCP_INFO */
+	case 2:  *info = tcp_info.__tcpi_unacked;      break;
+	case 3:  *info = tcp_info.__tcpi_sacked;       break;
+	case 4:  *info = tcp_info.__tcpi_lost;         break;
+	case 5:  *info = tcp_info.__tcpi_retrans;      break;
+	case 6:  *info = tcp_info.__tcpi_fackets;      break;
+	case 7:  *info = tcp_info.__tcpi_reordering;   break;
+# endif
+#endif // apple
+	default: return 0;
+	}
+
+	return 1;
+}
+#else
+static int tcp_get_info(struct connection *conn, long long int *info, int info_num)
+{
+	return 0;
+}
+#endif /* TCP_INFO */
+
 
 /*
  * Local variables:
