@@ -58,7 +58,7 @@ uint quic_cc_default_pacing_inter(const struct quic_cc *cc)
 }
 
 /* Returns true if congestion window on path ought to be increased. */
-int quic_cwnd_may_increase(const struct quic_cc_path *path)
+static int quic_cwnd_may_increase(const struct quic_cc_path *path)
 {
 	/* RFC 9002 7.8. Underutilizing the Congestion Window
 	 *
@@ -75,3 +75,32 @@ int quic_cwnd_may_increase(const struct quic_cc_path *path)
 	 */
 	return 2 * path->in_flight >= path->cwnd  || path->cwnd < 16384;
 }
+
+/* Restore congestion window for <path> to its minimal value. */
+void quic_cc_path_reset(struct quic_cc_path *path)
+{
+	path->cwnd = path->limit_min;
+}
+
+/* Set congestion window for <path> to <val>. Min and max limits are enforced. */
+void quic_cc_path_set(struct quic_cc_path *path, uint64_t val)
+{
+	path->cwnd = QUIC_MIN(val, path->limit_max);
+	path->cwnd = QUIC_MAX(path->cwnd, path->limit_min);
+
+	path->cwnd_last_max = QUIC_MAX(path->cwnd, path->cwnd_last_max);
+}
+
+/* Increment congestion window for <path> with <val>. Min and max limits are
+ * enforced. Contrary to quic_cc_path_set(), increase is performed only if a
+ * certain minimal level of the window was already filled.
+ */
+void quic_cc_path_inc(struct quic_cc_path *path, uint64_t val)
+{
+	if (quic_cwnd_may_increase(path)) {
+		path->cwnd = QUIC_MIN(path->cwnd + val, path->limit_max);
+		path->cwnd = QUIC_MAX(path->cwnd, path->limit_min);
+
+		path->cwnd_last_max = QUIC_MAX(path->cwnd, path->cwnd_last_max);
+	}
+}
