@@ -382,11 +382,7 @@ static inline void quic_cubic_update(struct quic_cc *cc, uint32_t acked)
 			inc = W_est_inc;
 	}
 
-	if (quic_cwnd_may_increase(path)) {
-		path->cwnd += inc;
-		path->cwnd = QUIC_MIN(path->max_cwnd, path->cwnd);
-		path->mcwnd = QUIC_MAX(path->cwnd, path->mcwnd);
-	}
+	quic_cc_path_inc(path, inc);
  leave:
 	TRACE_LEAVE(QUIC_EV_CONN_CC, cc->qc);
 }
@@ -433,7 +429,7 @@ static void quic_enter_recovery(struct quic_cc *cc)
 	}
 
 	c->ssthresh = (CUBIC_BETA_SCALED * path->cwnd) >> CUBIC_SCALE_FACTOR_SHIFT;
-	path->cwnd =  QUIC_MAX(c->ssthresh, (uint32_t)path->min_cwnd);
+	quic_cc_path_set(path, c->ssthresh);
 	c->state = QUIC_CC_ST_RP;
 	TRACE_LEAVE(QUIC_EV_CONN_CC, cc->qc, NULL, cc);
 }
@@ -455,10 +451,7 @@ static void quic_cc_cubic_ss_cb(struct quic_cc *cc, struct quic_cc_event *ev)
 			if (path->cwnd >= QUIC_CC_INFINITE_SSTHESH - acked)
 				goto out;
 
-			if (quic_cwnd_may_increase(path)) {
-				path->cwnd += acked;
-				path->mcwnd = QUIC_MAX(path->cwnd, path->mcwnd);
-			}
+			quic_cc_path_inc(path, acked);
 			quic_cc_hystart_track_min_rtt(cc, h, path->loss.latest_rtt);
 			if (ev->ack.pn >= h->wnd_end)
 				h->wnd_end = UINT64_MAX;
@@ -469,15 +462,11 @@ static void quic_cc_cubic_ss_cb(struct quic_cc *cc, struct quic_cc_event *ev)
 			}
 		}
 		else if (path->cwnd < QUIC_CC_INFINITE_SSTHESH - ev->ack.acked) {
-			if (quic_cwnd_may_increase(path)) {
-				path->cwnd += ev->ack.acked;
-				path->cwnd = QUIC_MIN(path->max_cwnd, path->cwnd);
-			}
+			quic_cc_path_inc(path, ev->ack.acked);
 		}
 		/* Exit to congestion avoidance if slow start threshold is reached. */
 		if (path->cwnd >= c->ssthresh)
 			c->state = QUIC_CC_ST_CA;
-		path->mcwnd = QUIC_MAX(path->cwnd, path->mcwnd);
 		break;
 
 	case QUIC_CC_EVT_LOSS:
@@ -550,10 +539,7 @@ static void quic_cc_cubic_cs_cb(struct quic_cc *cc, struct quic_cc_event *ev)
 		if (path->cwnd >= QUIC_CC_INFINITE_SSTHESH - acked)
 			goto out;
 
-		if (quic_cwnd_may_increase(path)) {
-			path->cwnd += acked;
-			path->mcwnd = QUIC_MAX(path->cwnd, path->mcwnd);
-		}
+		quic_cc_path_inc(path, acked);
 		quic_cc_hystart_track_min_rtt(cc, h, path->loss.latest_rtt);
 		if (quic_cc_hystart_may_reenter_ss(h)) {
 			/* Exit to slow start */
@@ -662,10 +648,10 @@ static void quic_cc_cubic_state_trace(struct buffer *buf, const struct quic_cc *
 	struct cubic *c = quic_cc_priv(cc);
 
 	path = container_of(cc, struct quic_cc_path, cc);
-	chunk_appendf(buf, " state=%s cwnd=%llu mcwnd=%llu ssthresh=%d rpst=%dms",
+	chunk_appendf(buf, " state=%s cwnd=%llu cwnd_last_max=%llu ssthresh=%d rpst=%dms",
 	              quic_cc_state_str(c->state),
 	              (unsigned long long)path->cwnd,
-	              (unsigned long long)path->mcwnd,
+	              (unsigned long long)path->cwnd_last_max,
 	              (int)c->ssthresh,
 	              !tick_isset(c->recovery_start_time) ? -1 :
 	              TICKS_TO_MS(tick_remain(c->recovery_start_time, now_ms)));
