@@ -661,7 +661,7 @@ static enum quic_rx_ret_frm qc_handle_crypto_frm(struct quic_conn *qc,
 	enum quic_rx_ret_frm ret = QUIC_RX_RET_FRM_DONE;
 	/* XXX TO DO: <cfdebug> is used only for the traces. */
 	struct quic_rx_crypto_frm cfdebug = {
-		.offset_node.key = crypto_frm->offset,
+		.offset_node.key = crypto_frm->offset_node.key,
 		.len = crypto_frm->len,
 	};
 	struct quic_cstream *cstream = qel->cstream;
@@ -670,10 +670,10 @@ static enum quic_rx_ret_frm qc_handle_crypto_frm(struct quic_conn *qc,
 
 	TRACE_ENTER(QUIC_EV_CONN_PRSHPKT, qc);
 
-	if (unlikely(crypto_frm->offset < cstream->rx.offset)) {
+	if (unlikely(crypto_frm->offset_node.key < cstream->rx.offset)) {
 		size_t diff;
 
-		if (crypto_frm->offset + crypto_frm->len <= cstream->rx.offset) {
+		if (crypto_frm->offset_node.key + crypto_frm->len <= cstream->rx.offset) {
 			/* Nothing to do */
 			TRACE_PROTO("Already received CRYPTO data",
 			            QUIC_EV_CONN_RXPKT, qc, pkt, &cfdebug);
@@ -684,10 +684,10 @@ static enum quic_rx_ret_frm qc_handle_crypto_frm(struct quic_conn *qc,
 		TRACE_PROTO("Partially already received CRYPTO data",
 		            QUIC_EV_CONN_RXPKT, qc, pkt, &cfdebug);
 
-		diff = cstream->rx.offset - crypto_frm->offset;
+		diff = cstream->rx.offset - crypto_frm->offset_node.key;
 		crypto_frm->len -= diff;
 		crypto_frm->data += diff;
-		crypto_frm->offset = cstream->rx.offset;
+		crypto_frm->offset_node.key = cstream->rx.offset;
 	}
 
 	if (!quic_get_ncbuf(ncbuf) || ncb_is_null(ncbuf)) {
@@ -696,7 +696,7 @@ static enum quic_rx_ret_frm qc_handle_crypto_frm(struct quic_conn *qc,
 	}
 
 	/* crypto_frm->offset > cstream-trx.offset */
-	off_rel = crypto_frm->offset - cstream->rx.offset;
+	off_rel = crypto_frm->offset_node.key - cstream->rx.offset;
 
 	/* RFC 9000 7.5. Cryptographic Message Buffering
 	 *
@@ -819,13 +819,12 @@ static inline unsigned int quic_ack_delay_ms(struct qf_ack *ack_frm,
 static int qc_parse_pkt_frms(struct quic_conn *qc, struct quic_rx_packet *pkt,
                              struct quic_enc_level *qel)
 {
-	struct list retry_frms = LIST_HEAD_INIT(retry_frms);
-	struct quic_frame *frm = NULL, *frm_tmp;
+	struct eb_root cf_frms_tree = EB_ROOT;
+	struct eb64_node *node;
+	struct quic_frame *frm = NULL;
 	const unsigned char *pos, *end;
 	enum quic_rx_ret_frm ret;
-	int fast_retrans = 0;
-	/* parsing may be rerun multiple times, but no more than <iter>. */
-	int iter = 3, parsing_stage = 0;
+	int fast_retrans = 0, cf_err = 0;
 
 	TRACE_ENTER(QUIC_EV_CONN_PRSHPKT, qc);
 	/* Skip the AAD */
@@ -903,36 +902,8 @@ static int qc_parse_pkt_frms(struct quic_conn *qc, struct quic_rx_packet *pkt,
 			break;
 		}
 		case QUIC_FT_CRYPTO:
-			ret = qc_handle_crypto_frm(qc, &frm->crypto, pkt, qel);
-			switch (ret) {
-			case QUIC_RX_RET_FRM_FATAL:
-				goto err;
-
-			case QUIC_RX_RET_FRM_AGAIN:
-				if (parsing_stage == 0) {
-					TRACE_STATE("parsing stage set to 1 (AGAIN encountered)", QUIC_EV_CONN_PRSHPKT, qc);
-					++parsing_stage;
-				}
-				/* Save frame in temp list to reparse it later. A new instance must be used for next packet frames. */
-				LIST_APPEND(&retry_frms, &frm->list);
-				frm = NULL;
-				break;
-
-			case QUIC_RX_RET_FRM_DUP:
-				if (qc_is_listener(qc) && qel == qc->iel &&
-				    !(qc->flags & QUIC_FL_CONN_HANDSHAKE_SPEED_UP)) {
-					fast_retrans = 1;
-				}
-				break;
-
-			case QUIC_RX_RET_FRM_DONE:
-				if (parsing_stage == 1) {
-					TRACE_STATE("parsing stage set to 2 (DONE after AGAIN)", QUIC_EV_CONN_PRSHPKT, qc);
-					++parsing_stage;
-				}
-				break;
-			}
-
+			eb64_insert(&cf_frms_tree, &frm->crypto.offset_node);
+			frm = NULL;
 			break;
 		case QUIC_FT_NEW_TOKEN:
 			if (qc_is_listener(qc)) {
@@ -1097,57 +1068,45 @@ static int qc_parse_pkt_frms(struct quic_conn *qc, struct quic_rx_packet *pkt,
 	if (frm)
 		qc_frm_free(qc, &frm);
 
-	while (!LIST_ISEMPTY(&retry_frms)) {
-		if (--iter <= 0) {
-			TRACE_ERROR("interrupt parsing due to max iteration reached",
-			            QUIC_EV_CONN_PRSHPKT, qc);
+	node = eb64_first(&cf_frms_tree);
+	while (node) {
+		frm = eb64_entry(node, struct quic_frame, crypto.offset_node);
+		/* only CRYPTO frames may be reparsed for now */
+		BUG_ON(frm->type != QUIC_FT_CRYPTO);
+		node = eb64_next(node);
+		ret = qc_handle_crypto_frm(qc, &frm->crypto, pkt, qel);
+		switch (ret) {
+		case QUIC_RX_RET_FRM_FATAL:
+			/* avoid freeing without eb_delete() */
+			frm = NULL;
 			goto err;
-		}
-		else if (parsing_stage <= 1) {
-			TRACE_ERROR("interrupt parsing due to buffering blocked on gap size limit",
-			            QUIC_EV_CONN_PRSHPKT, qc);
-			goto err;
-		}
 
-		parsing_stage = 0;
-		list_for_each_entry_safe(frm, frm_tmp, &retry_frms, list) {
-			/* only CRYPTO frames may be reparsed for now */
-			BUG_ON(frm->type != QUIC_FT_CRYPTO);
-			ret = qc_handle_crypto_frm(qc, &frm->crypto, pkt, qel);
-			switch (ret) {
-			case QUIC_RX_RET_FRM_FATAL:
-				goto err;
+		case QUIC_RX_RET_FRM_AGAIN:
+			TRACE_STATE("AGAIN encountered", QUIC_EV_CONN_PRSHPKT, qc);
+			cf_err = 1;
+			break;
 
-			case QUIC_RX_RET_FRM_AGAIN:
-				if (parsing_stage == 0) {
-					TRACE_STATE("parsing stage set to 1 (AGAIN encountered)", QUIC_EV_CONN_PRSHPKT, qc);
-					++parsing_stage;
-				}
-				break;
+		case QUIC_RX_RET_FRM_DONE:
+			TRACE_PROTO("frame handled", QUIC_EV_CONN_PRSAFRM, qc, frm);
+			break;
 
-			case QUIC_RX_RET_FRM_DONE:
-				TRACE_PROTO("frame handled after a new parsing iteration",
-				            QUIC_EV_CONN_PRSAFRM, qc, frm);
-				if (parsing_stage == 1) {
-					TRACE_STATE("parsing stage set to 2 (DONE after AGAIN)", QUIC_EV_CONN_PRSHPKT, qc);
-					++parsing_stage;
-				}
-				__fallthrough;
-			case QUIC_RX_RET_FRM_DUP:
-				qc_frm_free(qc, &frm);
-				break;
+		case QUIC_RX_RET_FRM_DUP:
+			if (qc_is_listener(qc) && qel == qc->iel &&
+				!(qc->flags & QUIC_FL_CONN_HANDSHAKE_SPEED_UP)) {
+				fast_retrans = 1;
 			}
+			break;
 		}
 
-		/* Always reset <frm> as it may be dangling after
-		 * list_for_each_entry_safe() usage. Especially necessary to
-		 * prevent a crash if loop is interrupted on max iteration.
-		 */
-		frm = NULL;
+		eb64_delete(&frm->crypto.offset_node);
+		qc_frm_free(qc, &frm);
 	}
 
+	if (cf_err)
+		goto err;
+
 	/* Error should be returned if some frames cannot be parsed. */
-	BUG_ON(!LIST_ISEMPTY(&retry_frms));
+	BUG_ON(!eb_is_empty(&cf_frms_tree));
 
 	if (fast_retrans && qc->iel && qc->hel) {
 		struct quic_enc_level *iqel = qc->iel;
@@ -1183,7 +1142,11 @@ static int qc_parse_pkt_frms(struct quic_conn *qc, struct quic_rx_packet *pkt,
  err:
 	if (frm)
 		qc_frm_free(qc, &frm);
-	list_for_each_entry_safe(frm, frm_tmp, &retry_frms, list) {
+	node = eb64_first(&cf_frms_tree);
+	while (node) {
+		frm = eb64_entry(node, struct quic_frame, crypto.offset_node);
+		node = eb64_next(node);
+		eb64_delete(&frm->crypto.offset_node);
 		qc_frm_free(qc, &frm);
 	}
 
@@ -1546,7 +1509,7 @@ static int qc_try_rm_hp(struct quic_conn *qc, struct quic_rx_packet *pkt,
 
 	ret = 1;
  out:
-	TRACE_LEAVE(QUIC_EV_CONN_TRMHP, qc);
+	TRACE_LEAVE(QUIC_EV_CONN_TRMHP, qc, pkt);
 	return ret;
 }
 
