@@ -141,7 +141,7 @@ static int qc_ssl_crypto_data_cpy(struct quic_conn *qc, struct quic_enc_level *q
 				goto leave;
 			}
 
-			frm->crypto.offset = cf_offset;
+			frm->crypto.offset_node.key = cf_offset;
 			frm->crypto.len = cf_len;
 			frm->crypto.qel = qel;
 			LIST_APPEND(&qel->pktns->tx.frms, &frm->list);
@@ -874,6 +874,6 @@ int qc_alloc_ssl_sock_ctx(struct quic_conn *qc)
 
  err:
 	TRACE_DEVEL("leaving on error", QUIC_EV_CONN_NEW, qc);
-	pool_free(pool_head_quic_ssl_sock_ctx, ctx);
+	qc_free_ssl_sock_ctx(&ctx);
 	goto leave;
 }
