@@ -124,7 +124,7 @@ void quic_cstream_free(struct quic_cstream *cs)
 
 	quic_free_ncbuf(&cs->rx.ncbuf);
 
-	qc_stream_desc_release(cs->desc, 0);
+	qc_stream_desc_release(cs->desc, 0, NULL);
 	pool_free(pool_head_quic_cstream, cs);
 }
 
@@ -143,7 +143,7 @@ struct quic_cstream *quic_cstream_new(struct quic_conn *qc)
 	}
 
 	cs->rx.offset = 0;
-	cs->rx.ncbuf = NCBMBUF_NULL;
+	cs->rx.ncbuf = NCBUF_NULL;
 	cs->rx.offset = 0;
 
 	cs->tx.offset = 0;
@@ -470,7 +470,16 @@ int quic_tls_derive_keys(const QUIC_AEAD *aead, const EVP_CIPHER *hp,
 {
 	size_t aead_keylen = (size_t)QUIC_AEAD_key_length(aead);
 	size_t aead_ivlen = (size_t)QUIC_AEAD_iv_length(aead);
+#ifdef QUIC_AEAD_API
+	size_t hp_len = 0;
+
+	if (hp == EVP_CIPHER_CHACHA20)
+		hp_len = 32;
+	else if (hp)
+		hp_len = (size_t)EVP_CIPHER_key_length(hp);
+#else
 	size_t hp_len = hp ? (size_t)EVP_CIPHER_key_length(hp) : 0;
+#endif
 
 	if (aead_keylen > keylen || aead_ivlen > ivlen || hp_len > hp_keylen)
 		return 0;
@@ -609,20 +618,28 @@ int quic_tls_rx_ctx_init(QUIC_AEAD_CTX **rx_ctx,
 	return 0;
 }
 
-/* Initialize <*aes_ctx> AES cipher context with <key> as key for encryption */
-int quic_tls_enc_aes_ctx_init(EVP_CIPHER_CTX **aes_ctx,
-                              const EVP_CIPHER *aes, unsigned char *key)
+/* Initialize <*hp_ctx> cipher context with <key> as key for header protection encryption */
+int quic_tls_enc_hp_ctx_init(EVP_CIPHER_CTX **hp_ctx,
+                              const EVP_CIPHER *hp, unsigned char *key)
 {
 	EVP_CIPHER_CTX *ctx;
 
+#ifdef QUIC_AEAD_API
+
+	if (hp == EVP_CIPHER_CHACHA20) {
+		*hp_ctx = EVP_CIPHER_CTX_CHACHA20;
+		return 1;
+	}
+#endif
+
 	ctx = EVP_CIPHER_CTX_new();
 	if (!ctx)
 		return 0;
 
-	if (!EVP_EncryptInit_ex(ctx, aes, NULL, key, NULL))
+	if (!EVP_EncryptInit_ex(ctx, hp, NULL, key, NULL))
 		goto err;
 
-	*aes_ctx = ctx;
+	*hp_ctx = ctx;
 	return 1;
 
  err:
@@ -630,17 +647,36 @@ int quic_tls_enc_aes_ctx_init(EVP_CIPHER_CTX **aes_ctx,
 	return 0;
 }
 
-/* Encrypt <inlen> bytes from <in> buffer into <out> with <ctx> as AES
+/* Encrypt <inlen> bytes from <in> buffer into <out> with <ctx> as
  * cipher context. This is the responsibility of the caller to check there
  * is at least <inlen> bytes of available space in <out> buffer.
  * Return 1 if succeeded, 0 if not.
  */
-int quic_tls_aes_encrypt(unsigned char *out,
+int quic_tls_hp_encrypt(unsigned char *out,
                          const unsigned char *in, size_t inlen,
-                         EVP_CIPHER_CTX *ctx)
+                         EVP_CIPHER_CTX *ctx, unsigned char *key)
 {
 	int ret = 0;
 
+#ifdef QUIC_AEAD_API
+
+	if (ctx == EVP_CIPHER_CTX_CHACHA20) {
+		uint32_t counter;
+		/* According to RFC 9001, 5.4.4. ChaCha20-Based Header Protection:
+		 * The first 4 bytes of the sampled ciphertext are the block counter.
+		 * The remaining 12 bytes are used as the nonce.
+		 */
+#if __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
+		counter = (uint32_t)in[0] + (uint32_t)(in[1] << 8) + (uint32_t)(in[2] << 16) + (uint32_t)(in[3] << 24);
+#else
+		memcpy(&counter, in, sizeof(counter));
+#endif
+		CRYPTO_chacha_20(out, out, inlen, key, in + sizeof(counter), counter);
+		return 1;
+	}
+
+#endif
+
 	if (!EVP_EncryptInit_ex(ctx, NULL, NULL, NULL, in) ||
 	    !EVP_EncryptUpdate(ctx, out, &ret, out, inlen) ||
 	    !EVP_EncryptFinal_ex(ctx, out, &ret))
@@ -649,20 +685,28 @@ int quic_tls_aes_encrypt(unsigned char *out,
 	return 1;
 }
 
-/* Initialize <*aes_ctx> AES cipher context with <key> as key for decryption */
-int quic_tls_dec_aes_ctx_init(EVP_CIPHER_CTX **aes_ctx,
-                              const EVP_CIPHER *aes, unsigned char *key)
+/* Initialize <*hp_ctx> cipher context with <key> as key for header protection decryption */
+int quic_tls_dec_hp_ctx_init(EVP_CIPHER_CTX **hp_ctx,
+                              const EVP_CIPHER *hp, unsigned char *key)
 {
 	EVP_CIPHER_CTX *ctx;
 
+#ifdef QUIC_AEAD_API
+
+	if (hp == EVP_CIPHER_CHACHA20) {
+		*hp_ctx = EVP_CIPHER_CTX_CHACHA20;
+		return 1;
+	}
+#endif
+
 	ctx = EVP_CIPHER_CTX_new();
 	if (!ctx)
 		return 0;
 
-	if (!EVP_DecryptInit_ex(ctx, aes, NULL, key, NULL))
+	if (!EVP_DecryptInit_ex(ctx, hp, NULL, key, NULL))
 		goto err;
 
-	*aes_ctx = ctx;
+	*hp_ctx = ctx;
 	return 1;
 
  err:
@@ -670,17 +714,36 @@ int quic_tls_dec_aes_ctx_init(EVP_CIPHER_CTX **aes_ctx,
 	return 0;
 }
 
-/* Decrypt <in> data into <out> with <ctx> as AES cipher context.
+/* Decrypt <in> data into <out> with <ctx> as cipher context.
  * This is the responsibility of the caller to check there is at least
  * <outlen> bytes into <in> buffer.
  * Return 1 if succeeded, 0 if not.
  */
-int quic_tls_aes_decrypt(unsigned char *out,
+int quic_tls_hp_decrypt(unsigned char *out,
                          const unsigned char *in, size_t inlen,
-                         EVP_CIPHER_CTX *ctx)
+                         EVP_CIPHER_CTX *ctx, unsigned char *key)
 {
 	int ret = 0;
 
+#ifdef QUIC_AEAD_API
+	if (ctx == EVP_CIPHER_CTX_CHACHA20) {
+		uint32_t counter;
+
+		/* According to RFC 9001, 5.4.4. ChaCha20-Based Header Protection:
+		 * The first 4 bytes of the sampled ciphertext are the block counter.
+		 * The remaining 12 bytes are used as the nonce.
+		 */
+#if __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
+		counter = (uint32_t)in[0] + (uint32_t)(in[1] << 8) + (uint32_t)(in[2] << 16) + (uint32_t)(in[3] << 24);
+#else
+		memcpy(&counter, in, sizeof(counter));
+#endif
+		CRYPTO_chacha_20(out, out, inlen, key, in + sizeof(counter), counter);
+		return 1;
+	}
+
+#endif
+
 	if (!EVP_DecryptInit_ex(ctx, NULL, NULL, NULL, in) ||
 	    !EVP_DecryptUpdate(ctx, out, &ret, out, inlen) ||
 	    !EVP_DecryptFinal_ex(ctx, out, &ret))
